<?php
namespace Drupal\aggrid\Commands;

use Drush\Commands\DrushCommands;

class aggridDrushCommands extends DrushCommands {
  /**
   * Downloads the most current ag-Grid Library files
   *
   *   Argument provided to the drush command.
   *
   * @command aggrid:download
   * @aliases agg-download
   * @usage aggrid:download
   *   Download/Update the ag-Grid Library from GitHub
   */
  public function download() {
    // Create a file system service manager.
    $fileSystem = \Drupal::service('file_system');

    // Remove the existing directory if it exists.
    $library_directory = $fileSystem->realpath('libraries/');
    $aggrid_library_directory = $library_directory .'/ag-grid';
    if (file_exists($library_directory) && file_exists($aggrid_library_directory)) {
      // Remove the existing file.
      if (file_exists($aggrid_library_directory . '/ag-grid-community.min.js')) {
        $fileSystem->unlink($aggrid_library_directory . '/ag-grid-community.min.js');
      }
      if (file_exists($aggrid_library_directory . '/ag-grid-enterprise.min.js')) {
        $fileSystem->unlink($aggrid_library_directory . '/ag-grid-enterprise.min.js');
      }
      $fileSystem->rmdir($aggrid_library_directory);
    }

    // Create the directory(s).
    if (!file_exists($library_directory)) {
      $fileSystem->mkdir('libraries');
    }
    $fileSystem->mkdir('libraries/ag-grid');

    // Download the community file.
    if (drush_shell_exec('wget https://github.com/ag-grid/ag-grid/raw/master/packages/ag-grid-community/dist/ag-grid-community.min.js  -O ' . $fileSystem->realpath('libraries/ag-grid') . '/ag-grid-community.min.js')) {
      drush_log(dt('ag-Grid Community library has been successfully installed at libraries/ag-grid', [], 'success'), 'success');
    }
    else {
      drush_log(dt('Error: unable to install ag-Grid Community library', [], 'error'), 'error');
    }

    // Download the enterprise file.
    if (drush_shell_exec('wget https://github.com/ag-grid/ag-grid/raw/master/packages/ag-grid-enterprise/dist/ag-grid-enterprise.min.js  -O ' . $fileSystem->realpath('libraries/ag-grid') . '/ag-grid-enterprise.min.js')) {
      drush_log(dt('ag-Grid Enterprise library has been successfully installed at libraries/ag-grid', [], 'success'), 'success');
    }
    else {
      drush_log(dt('Error: unable to install ag-Grid Enterprise library', [], 'error'), 'error');
    }
  }
}