<?php

namespace Drupal\aggrid\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Html;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\aggrid\Entity;
use Drupal\Core\Form\ConfigFormBase;

/**
 * Plugin implementation of the 'aggrid_formatter_type' formatter.
 *
 * @FieldFormatter(
 *   id = "html_formatter_type",
 *   label = @Translation("HTML grid view mode"),
 *   field_types = {
 *     "aggrid"
 *   }
 * )
 */
class htmlFormatterType extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        // Implement default settings.
      ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    return [
        // Implement settings form.
      ] + parent::settingsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    // Implement settings summary.

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    foreach ($items as $delta => $item) {
      $config = \Drupal::config('aggrid.general');
      $field_settings = $this->getFieldSettings();

      $field_name = $this->fieldDefinition->getName();

      $item_id = Html::getUniqueId("ht-$field_name-$delta");

      $aggridEntity = Entity\aggrid::load($items[$delta]->aggrid_id);
      $aggridDefault = json_decode($aggridEntity->get('aggridDefault'));

      if ($items[$delta]->value != '') {
        $aggridValue = json_decode($items[$delta]->value);
      } else {
        $aggridValue = $aggridDefault->rowData;
      }

      $columns[][][][] = "";
      $columnFields = "";
      $rowData = $aggridValue;

      // Build table
      $rowIndex = 0;
      $colIndex = 0;

      foreach($aggridDefault->columnDefs as $column) {
        $rowIndex = $rowIndex > 1 ? $rowIndex : 1;
        $colIndex++;

        $columns[1][$colIndex][$column->headerName] = [];
        $columns[1][$colIndex][$column->headerName]['headerName'] = $column->headerName;
        $columns[1][$colIndex][$column->headerName]['field'] = isset($column->field) ? $column->field : NULL;

        // Set the field if available
        if (isset($column->field)) {
          $columnFields .= $column->field .",";
        }

        if (isset($column->children)) { // If children, then dive down for headers, otherwise establish column
          $columns[1][$colIndex][$column->headerName]['colspan'] = count($column->children);
          foreach ($column->children as $child) {
            $rowIndex = $rowIndex > 2 ? $rowIndex : 2;

            $columns[2][$colIndex][$column->headerName .' - '. $child->headerName] = [];
            $columns[2][$colIndex][$column->headerName .' - '. $child->headerName]['headerName'] = $child->headerName;
            $columns[2][$colIndex][$column->headerName .' - '. $child->headerName]['field'] = isset($child->field) ? $child->field : NULL;

            // Set the field if available
            if (isset($child->field)) {
              $columnFields .= $child->field .",";
            }

            if (isset($child->children)) {
              $columns[2][$colIndex][$column->headerName .' - '. $column->headerName]['colspan'] = count($child->children);
              foreach ($child->children as $subchild) {
                $rowIndex = $rowIndex > 3 ? $rowIndex : 3;

                $columns[3][$colIndex][$column->headerName .' - '. $child->headerName .' - '. $subchild->headerName] = [];
                $columns[3][$colIndex][$column->headerName .' - '. $child->headerName .' - '. $subchild->headerName]['headerName'] = $subchild->headerName;
                $columns[3][$colIndex][$column->headerName .' - '. $child->headerName .' - '. $subchild->headerName]['field'] = isset($subchild->field) ? $subchild->field : NULL;
                $columns[3][$colIndex][$column->headerName .' - '. $child->headerName .' - '. $subchild->headerName]['colspan'] = 1;

                // Set the field if available
                if (isset($subchild->field)) {
                  $columnFields .= $subchild->field .",";
                }
              }
            } else {
              // Just one for colspan
              $columns[2][$colIndex][$column->headerName .' - '. $child->headerName]['colspan'] = 1;
            }
          }
        } else {
          // Just one for colspan
          $columns[1][$colIndex][$column->headerName]['colspan'] = 1;
        }
      }

      // Put columnFields to headers, trim comma, and put to array
      $headers = $columnFields;
      $headers = substr($headers, 0, strlen($headers)-1);
      $headers = str_getcsv($headers);

      // Build table
      $table_render = '';
      $table_render .= '<table><thead>';

      // Get the header rows
      for ($y = 1; $y <= $rowIndex; $y++) {
        // Each header row and each column cell with spanning
        $table_render .= '<tr>';
        for ($x = 1; $x <= $colIndex; $x++) {
          if (!array_key_exists($x, $columns[$y])) {
            $table_render .= '<th></th>';
          } else {
            foreach ($columns[$y][$x] as $column => $value) {
              $table_render .= '<th colspan="' . $columns[$y][$x][$column]['colspan'] . '">' . $columns[$y][$x][$column]['headerName'] . '</th>';
            }
          }
        }
        $table_render .= '</tr>';
      }
      // Close up the headers and start on data rows
      $table_render .= '</thead><tbody>';

      // Data rows
      for ($i = 0; $i < count($rowData); $i++) {
        // each row... then each cell in each row
        $table_render .= '<tr>';
        foreach($headers as $field) {
          // Loop and look for cell data
          if (property_exists($rowData[$i], $field)) {
            // has data, put it to cell
            $table_render .= '<td>'. $rowData[$i]->$field .'</td>';
          } else {
            // no data, just a blank cell
            $table_render .= '<td></td>';
          }
        }
        // close up the row
        $table_render .= '</tr>';
      }

      // Close up the table
      $table_render .= '</tbody></table>';

      $elements[$delta]['container'] = [
        '#suffix' => $table_render,
        '#attributes' => ['id' => [$item_id .'_table']],
      ];

      /*
       * Putting this code to the side for now. They're currently working on multiple headers
       * for the '#type' => 'table'
       *
       *

      // Loop through header array and dive down max 3 header rows. Squash all down to single row with only the items with fields.
      // Header 1
      foreach($aggridDefault->columnDefs as $column) {
        if (isset($column->children)) { // If children, then dive down for headers, otherwise establish column
          foreach ($column->children as $child) {
            // Header 2
            if (isset($child->children)) {
              foreach ($child->children as $subchild) {
                if (isset($subchild->field)) {
                  // Header from row 3
                  $columns[$column->headerName . ' - ' . $child->headerName . ' - ' . $subchild->headerName] = [];
                  $columns[$column->headerName . ' - ' . $child->headerName . ' - ' . $subchild->headerName]['field'] = $subchild->field;
                }
              }
            } else {
              if (isset($child->field)) {
                // Header from row 2
                $columns[$column->headerName . ' - ' . $child->headerName] = [];
                $columns[$column->headerName . ' - ' . $child->headerName]['field'] = $child->field;
              }
            }
          }
      } else {
          if (isset($column->field)) {
            // Header from row 1
            $columns[$column->headerName] = [];
            $columns[$column->headerName]['field'] = $column->field;
          }
        }
      }

      // Headers
      foreach($columns as $column => $value) {
        array_push($header, $column);
      }
      // Row Data
      for ($i = 0; $i < count($rowData); $i++) {
        foreach($columns as $column => $value) {
          $colField = $columns[$column]['field'];
          $tabledata[$i][$columns[$column]['field']] = [
            'data' => $rowData[$i]->$colField,
            'class' => ['row_' . $columns[$column]['field'], 'col_' . $columns[$column]['field']],
          ];
        }
      }

      $elements[$delta]['tablefield'] = [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $tabledata,
        '#attributes' => [
          'id' => 'tablefield-' . $delta,
          'class' => [
            'tablefield',
          ],
        ],
        '#prefix' => '<div id="tablefield-wrapper-' . $delta . '" class="tablefield-wrapper">',
        '#suffix' => '</div>',
      ];

      */

    }

    return $elements;
  }

  /**
   * Generate the output appropriate for one field item.
   *
   * @param \Drupal\Core\Field\FieldItemInterface $item
   *   One field item.
   *
   * @return string
   *   The textual output generated.
   */
  protected function viewValue(FieldItemInterface $item) {
    // The text value has no text format assigned to it, so the user input
    // should equal the output, including newlines.
    return nl2br(Html::escape($item->value));
  }

}
