<?php

namespace Drupal\aggrid\Plugin\Field\FieldWidget;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\Html;
use Drupal\aggrid\Entity;
use Drupal\Core\Form\ConfigFormBase;

/**
 * Plugin implementation of the 'aggrid' widget.
 *
 * @FieldWidget(
 *   id = "aggrid_widget_type",
 *   label = @Translation("ag-Grid edit mode"),
 *   field_types = {
 *     "aggrid"
 *   }
 * )
 */
class aggridWidgetType extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    /*\\
    $summary[] = t('Textfield size: @size', ['@size' => $this->getSetting('size')]);
    if (!empty($this->getSetting('placeholder'))) {
      $summary[] = t('Placeholder: @placeholder', ['@placeholder' => $this->getSetting('placeholder')]);
    }
    */
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state)
  {
    $config = \Drupal::config('aggrid.general');
    //$field_settings = $this->getFieldSettings();
    //$settings = $this->getSettings();

    $field_name = $this->fieldDefinition->getName();

    $item_id = Html::getUniqueId("ht-$field_name-$delta");

    if (isset($form['#parents'][0]) && $form['#parents'][0] == 'default_value_input') {

      $options = [];

      $aggridEntities = \Drupal::entityManager()->getStorage('aggrid')->loadMultiple();

      foreach ($aggridEntities as $aggridEntity) {
        $options[$aggridEntity->id()] = $aggridEntity->label();
      }

      $element['aggrid_id'] = [
        '#type' => 'select',
        '#empty_option' => t(' - Select - '),
        '#options' => $options,
        '#title' => t('ag-Grid Default Config Entity'),
        '#description' => t('Choose ag-Grid Config Entity for Default. *Once saved, this cannot be modified.'),
        '#default_value' => isset($items[$delta]->aggrid_id) ? $items[$delta]->aggrid_id : NULL,
        '#required' => TRUE,
      ];

      $widget['value'] = [
        '#type' => 'hidden',
        '#attributes' => ['id' => [$item_id]],
        '#default_value' => NULL,
      ];

    } else {

      $element['container'] = [
        '#suffix' => $items[$delta]->description.' <div class="aggrid-widget ag-theme-balham" id="' . $item_id . '_aggrid" data-license="'. $config->get('license_key') .'" data-edit="true" data-target="' . $item_id . '"></div><div id="' . $item_id . '_error" title="Validation Error(s)"  hidden="hidden"></div>',
        '#attributes' => ['class' => ['aggrid-widget']],
        '#attached' => [
          'library' => [
            'aggrid/widget',
          ],
        ],
      ];
      if ($config->get('version') == "Enterprise") {
        array_push($element['container']['#attached']['library'], 'aggrid/ag-grid-enterprise');
      } else {
        array_push($element['container']['#attached']['library'], 'aggrid/ag-grid-community');
      }

      $aggridEntity = Entity\aggrid::load($items[$delta]->aggrid_id);
      $aggridDefault = json_decode($aggridEntity->get('aggridDefault'));

      if ($items[$delta]->value != '') {
        $aggridValue = $items[$delta]->value;
      } else {
        $aggridValue = json_encode($aggridDefault->rowData);
      }

      $widget['aggrid_id'] = [
        '#type' => 'hidden',
        '#default_value' => isset($items[$delta]->aggrid_id) ? $items[$delta]->aggrid_id : NULL,
      ];

      $widget['columnDefs'] = [
        '#type' => 'hidden',
        '#attributes' => ['id' => [$item_id .'_columnDefs']],
        '#default_value' => Xss::filterAdmin(json_encode($aggridDefault->columnDefs)),
      ];

      if (isset($aggridDefault->rowSettings)) {
        $widget['rowSettings'] = [
          '#type' => 'hidden',
          '#attributes' => ['id' => [$item_id . '_rowSettings']],
          '#default_value' => json_encode($aggridDefault->rowSettings),
        ];
      }

      $widget['value'] = [
        '#type' => 'hidden',
        '#attributes' => ['id' => [$item_id.'_rowData']],
        '#default_value' => Xss::filter($aggridValue),
      ];

      $widget['addOptions'] = [
        '#type' => 'hidden',
        '#attributes' => ['id' => [$item_id.'_addOptions']],
        '#default_value' => Xss::filterAdmin($aggridEntity->get('addOptions')),
      ];
    }

    $element = $element + $widget;
    return $element;
  }

}
